@@ -2,11 +2,14 @@
 import asyncio
 import logging
 import re
+import time
+import urllib.parse
 from dataclasses import dataclass, field
 
 import aiohttp
 
-from .const import CONNECTION_ERROR, DATA_ERROR, MAC_REGEX  # , test_response
+from .const import MAC_REGEX  # , test_response
+from .const import CONNECTION_ERROR, DATA_ERROR, MAC_VENDOR
 
 _LOGGER = logging.getLogger(__name__)
 
@@ -27,6 +30,7 @@ class SkyQHub:
         self._connection_failed = False
         self._dataparse_failed = False
         self.success_init = False
+        self._macvendorlist = {}
 
     async def async_connect(self):
         """Test the router is accessible."""
@@ -37,7 +41,9 @@ class SkyQHub:
         """Retrieve data from Sky Hub and return parsed result."""
         parseddata = None
         try:
-            async with getattr(self.websession, "get")(self.url,) as response:
+            async with getattr(self.websession, "get")(
+                self.url,
+            ) as response:
                 if response.status == HTTP_OK:
                     if self._connection_failed:
                         self._log_message(
@@ -48,7 +54,7 @@ class SkyQHub:
                         )
                     responsedata = await response.text()
                     # responsedata = test_response
-                    parseddata = _parse_skyhub_response(responsedata)
+                    parseddata = await self._async_parse_skyhub_response(responsedata)
                     if self._dataparse_failed:
                         self._log_message(
                             "Response data from Sky Hub corrected",
@@ -78,10 +84,57 @@ class SkyQHub:
             else:
                 message = f"Invalid response from Sky Hub: {err}"
             self._log_message(
-                message, level=ERROR, error_type=DATA_ERROR,
+                message,
+                level=ERROR,
+                error_type=DATA_ERROR,
             )
             return
 
+    async def _async_parse_skyhub_response(self, data_str):
+        """Parse the Sky Hub data format."""
+        pattmatch = re.search("attach_dev = '(.*)'", data_str)
+        if pattmatch is None:
+            raise OSError(
+                "Error: Impossible to fetch data from Sky Hub. Try to reboot the router."
+            )
+        patt = pattmatch.group(1)
+
+        dev = [patt1.split(",") for patt1 in patt.split("<lf>")]
+
+        devices = []
+        devlen = len(dev)
+        i = 0
+        for dvc in dev:
+            i += 1
+            if MAC_REGEX.match(dvc[1]):
+                mac = dvc[1]
+                name = dvc[0]
+                if mac in self._macvendorlist:
+                    vendor = self._macvendorlist[mac]
+                else:
+                    vendor = await self._async_get_macvendor(mac)
+                    if i < devlen:
+                        time.sleep(1)
+                devices.append(_Device(mac, name, vendor))
+            else:
+                raise RuntimeError(
+                    f"Error: MAC address {dvc[1]} not in correct format."
+                )
+
+        return devices
+
+    async def _async_get_macvendor(self, mac):
+        try:
+            async with getattr(self.websession, "get")(
+                MAC_VENDOR.format(urllib.parse.quote(mac))
+            ) as response:
+                if response.status == HTTP_OK:
+                    responsedata = await response.text()
+                    self._macvendorlist[mac] = responsedata
+                    return responsedata
+        except Exception:
+            return None
+
     def _log_message(
         self, log_message, unset_error=False, level=ERROR, error_type=None
     ):
@@ -108,31 +161,11 @@ class SkyQHub:
         return
 
 
-def _parse_skyhub_response(data_str):
-    """Parse the Sky Hub data format."""
-    pattmatch = re.search("attach_dev = '(.*)'", data_str)
-    if pattmatch is None:
-        raise OSError(
-            "Error: Impossible to fetch data from Sky Hub. Try to reboot the router."
-        )
-    patt = pattmatch.group(1)
-
-    dev = [patt1.split(",") for patt1 in patt.split("<lf>")]
-
-    devices = []
-    for dvc in dev:
-        if MAC_REGEX.match(dvc[1]):
-            devices.append(_Device(dvc[1], dvc[0]))
-        else:
-            raise RuntimeError(f"Error: MAC address {dvc[1]} not in correct format.")
-
-    return devices
-
-
 @dataclass
 class _Device:
     mac: str = field(init=True, repr=True, compare=True)
     name: str = field(init=True, repr=True, compare=True)
+    vendor: str = field(init=True, repr=True, compare=True)
 
     def asdict(self):
-        return {"mac": self.mac}
+        return {"mac": self.mac, "vendor": self.vendor}
